/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.bpmn.views {
import flash.display.DisplayObject;
import flash.events.Event;

import mx.collections.ListCollectionView;

import mx.containers.Canvas;
import mx.core.EventPriority;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;

import org.ow2.orchestra.designer.bpmn.model.IDiagramModel;
import org.ow2.orchestra.designer.bpmn.views.process.ProcessPoolView;

/**
 * @author Loic Albertin
 */
public class AbstractDiagramView extends Canvas{
  [Bindable]
  private var _flowModel:IDiagramModel;

  public function AbstractDiagramView(flowModel:IDiagramModel) {
    _flowModel = flowModel;

      this.automationName = "view" + _flowModel.id;

      //Add manually in the processView all elements already containing in the processModel
      // Then, next added elements will be automatically add in the processView by event
      var elements:ListCollectionView = _flowModel.elements;
      for each (var element:Object in elements) {
        addElementToProcessView(element);
      }
      // Add event listener with weakreference: the listener will be removed when the processview is removed (and garbage collected)
      _flowModel.elements.addEventListener(CollectionEvent.COLLECTION_CHANGE, modelElementsChanged, false, EventPriority.DEFAULT, true);

  }

  public function get flowModel():IDiagramModel {
    return _flowModel;
  }


    /**
     *
     * @param event
     */
    public function modelElementsChanged(event:Event):void {
      // event instance of CollectionEvent
      // check kind attribute for "add, delete, move..."
      // get added or removed child
      if (event is CollectionEvent) {
        var collectionEvent:CollectionEvent = event as CollectionEvent;
        // model elements changed.
        var changedElements:Array = collectionEvent.items;

        if (collectionEvent.kind == CollectionEventKind.ADD) {
          for each (var addedElement:Object in changedElements) {
            addElementToProcessView(addedElement);
          }
        } else if (collectionEvent.kind == CollectionEventKind.REMOVE) {
          for each (var removedElement:Object in changedElements) {
            if (removedElement is AbstractElement) {
              var viewToRemove:AbstractElementView = getElementViewByModel(removedElement as AbstractElement);
              if (viewToRemove != null) {
                removeElementView(viewToRemove);
              }
            }
          }
        } else if (collectionEvent.kind == CollectionEventKind.RESET) {
          for each (var child:DisplayObject in getChildren()) {
            if (!(child is ProcessPoolView) && child is AbstractElementView) {
              removeElementView(child as AbstractElementView);
            }
          }
        }
      }
    }

    private function removeElementView(viewToRemove:AbstractElementView):void {
      this.removeChild(viewToRemove);
      viewToRemove.dispose();
    }

    /**
     * Add an element to the processView
     */
    private function addElementToProcessView(elementToAdd:Object):void {
      if (elementToAdd is AbstractElement) {
        var elementView:AbstractElementView = ViewFactory.getViewByModel(elementToAdd as AbstractElement, this);
        if (elementView != null) {
          this.addChild(elementView);
        }
      }
    }

    /**
     * Get element view linked to the given model
     * @param elementModel
     * @return the view linked to the model
     */
    public function getElementViewByModel(elementModel:AbstractElement):AbstractElementView {
      var result:AbstractElementView = null;

      for each (var child:DisplayObject in this.getChildren()) {
        if (child is AbstractElementView) {
          if ((child as AbstractElementView).model == elementModel) {
            result = (child as AbstractElementView);
            break;
          }
        }
      }

      return result;
    }
  }
}
