/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.bpmn.views.data {
import flash.events.Event;

import mx.binding.utils.ChangeWatcher;
import mx.containers.HBox;
import mx.controls.Alert;
import mx.controls.Label;
import mx.events.CollectionEvent;
import mx.events.ResizeEvent;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;

import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionOperationModel;
import org.ow2.orchestra.designer.bpmn.model.process.AbstractBPMNElementWithPosition;

import org.ow2.orchestra.designer.bpmn.model.process.SequenceFlowModel;
import org.ow2.orchestra.designer.bpmn.views.*;

public class DataFlowView extends FlowView {
  private var positionLabel:Label;
  private var positionBox:HBox;

  function DataFlowView(flowModel:FlowModel, diagramView:AbstractDiagramView) {
    positionLabel = new Label();
    positionBox = new HBox();
    positionBox.setStyle("backgroundColor", "#000000");
    positionBox.setStyle("color", "#ffffff");
    positionBox.setStyle("cornerRadius", "10");
    positionBox.setStyle("borderStyle", "solid");
    positionBox.addChild(positionLabel);
    positionBox.addEventListener(ResizeEvent.RESIZE, setPositionBoxPosition);

    super(flowModel, diagramView);

    flowModel.targetElement.incomingFlows.addEventListener(CollectionEvent.COLLECTION_CHANGE, redraw);
  }

  public override function redraw(event:Event = null, retries:int = 10):void {
    super.redraw(event, retries);

    // draw position
    if (FlowModel(model).targetElement is ExpressionOperationModel) {
      var pos:int = FlowModel(model).targetElement.incomingFlows.getItemIndex(model);
      positionLabel.text =  " " + String.fromCharCode(pos + "a".charCodeAt()) + " ";
      positionBox.setVisible(true);
      setPositionBoxPosition();
      canvas.addChild(positionBox);
    } else {
      positionBox.setVisible(false);
    }
  }


  override protected function modelChanged(event:Event = null):void {
    // Use target view for listeners because the targetView still reference the old target model.
    if (targetView != null) {
      AbstractElementWithPosition(targetView.model).incomingFlows.removeEventListener(CollectionEvent.COLLECTION_CHANGE, redraw);
    }
    super.modelChanged(event);
    // Use target view for listeners because the targetView now reference the new target model.
    if (targetView != null) {
      AbstractElementWithPosition(targetView.model).incomingFlows.addEventListener(CollectionEvent.COLLECTION_CHANGE, redraw);
    }
  }

  private function setPositionBoxPosition(event:Event = null):void {
    positionBox.x = (_sourcePoint.x + _targetPoint.x - positionBox.width) / 2;
    positionBox.y = (_sourcePoint.y + _targetPoint.y - positionBox.height) / 2;
  }

  override public function dispose():void {
    super.dispose();
    FlowModel(model).targetElement.incomingFlows.addEventListener(CollectionEvent.COLLECTION_CHANGE, redraw);
  }
}
}
