package org.ow2.orchestra.designer.bpmn.views.utils {
  import flash.display.Graphics;
  import flash.geom.Point;

  /**
   *
   * Copyright (c) 2008 Noel Billig (www.dncompute.com)
   *
   * Permission is hereby granted, free of charge, to any person obtaining a copy
   * of this software and associated documentation files (the "Software"), to deal
   * in the Software without restriction, including without limitation the rights
   * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   * copies of the Software, and to permit persons to whom the Software is
   * furnished to do so, subject to the following conditions:
   *
   * The above copyright notice and this permission notice shall be included in
   * all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   * THE SOFTWARE.
   */


  public class ArrowUtil {


    public static function drawArrow(graphics:Graphics, start:Point, end:Point, style:ArrowStyle = null, clearGraphics:Boolean = true):void {

      if (start.equals(end)) {
        return;
      }

      var arrowStyle:ArrowStyle;
      if (style == null) {
        arrowStyle = new ArrowStyle();
      } else {
        arrowStyle = style;
      }

      if (clearGraphics) {
        graphics.clear();
      }
      graphics.lineStyle(arrowStyle.thickness, arrowStyle.color);

      var halfWidth:Number = (arrowStyle.headWidth != -1) ? arrowStyle.headWidth / 2 : arrowStyle.headLength / 2;
      var fullVect:Point = end.subtract(start);

      // if default transition
      if (arrowStyle.isDefault) {
        var defPnt:Point = fullVect.clone();
        defPnt.normalize(arrowStyle.headLength);
        defPnt = defPnt.add(start);

        var defVect:Point = new Point(fullVect.x - fullVect.y, fullVect.x + fullVect.y);
        defVect.normalize(halfWidth);
        var defPnt1:Point = defPnt.add(defVect);
        var defPnt2:Point = defPnt.subtract(defVect);

        graphics.moveTo(defPnt1.x, defPnt1.y);
        graphics.lineTo(defPnt2.x, defPnt2.y);
      } else if (arrowStyle.isConditional) {
        // if conditional
        var condEndPnt:Point = fullVect.clone();
        condEndPnt.normalize(arrowStyle.headLength);
        condEndPnt = condEndPnt.add(start);

        var condMidPnt:Point = Point.interpolate(start, condEndPnt, 0.5);
        var condVect:Point = new Point(fullVect.y, - fullVect.x);
        condVect.normalize(halfWidth);
        var condPnt1:Point = condMidPnt.add(condVect);
        var condPnt2:Point = condMidPnt.subtract(condVect);

        graphics.beginFill(0xffffff);
        graphics.moveTo(start.x, start.y);
        graphics.lineTo(condPnt1.x, condPnt1.y);
        graphics.lineTo(condEndPnt.x, condEndPnt.y);
        graphics.lineTo(condPnt2.x, condPnt2.y);
        graphics.lineTo(start.x, start.y);
        graphics.endFill();
        start = condEndPnt;
      } else if (arrowStyle.isMessage) {
        // if message, draw circle at start
        graphics.drawCircle(start.x, start.y, arrowStyle.headLength / 4);
        start = Point.interpolate(end, start, arrowStyle.headLength / (4 * Point.distance(start, end)));
      }

      if (arrowStyle.headFill) {
        graphics.beginFill(arrowStyle.color);
      } else {
        graphics.beginFill(0xffffff);
      }

      fullVect = end.subtract(start);


      //figure out where the arrow head starts
      var headPnt:Point = fullVect.clone();
      headPnt.normalize(headPnt.length - arrowStyle.headLength);
      headPnt = headPnt.add(start);

      //calculate the arrowhead corners
      var headPntNorm:Point = new Point(fullVect.y, -fullVect.x);
      headPntNorm.normalize(halfWidth);
      var edge1:Point = headPnt.add(headPntNorm);
      var edge2:Point = headPnt.subtract(headPntNorm);


      if (!arrowStyle.isDashed) {
        graphics.moveTo(start.x, start.y);
        graphics.lineTo(headPnt.x, headPnt.y);
      } else {
        var incr:Number = 5 / Point.distance(start, headPnt);
        for (var i:Number = 0; i < 1; i += 2 * incr) {
          var p1:Point = Point.interpolate(headPnt, start, i);
          var p2:Point = Point.interpolate(headPnt, start, Math.min(1, i + incr));
          graphics.moveTo(p1.x, p1.y);
          graphics.lineTo(p2.x, p2.y);
          graphics.moveTo(p2.x, p2.y);
        }
        graphics.lineTo(headPnt.x, headPnt.y);
      }
      graphics.lineTo(edge1.x, edge1.y);
      graphics.lineTo(end.x, end.y);
      graphics.lineTo(edge2.x, edge2.y);
      graphics.lineTo(headPnt.x, headPnt.y);

      graphics.endFill();


    }
  }
}
